package com.yem.em.task;

import com.yem.em.utils.AtlasUtils;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.exception.KDException;
import kd.bos.schedule.executor.AbstractTask;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.util.Date;
import java.util.Map;

import static com.yem.em.task.MatchingPLMChangeUpdateTaskEdit.queryPLMChange;

/**
 * @Description: 1、如《PLM变更单》分录【替换关系】=【相互替换】【新替换旧】【互不替换】,分录【启用停用状态】=‘停用’，并且【替换件启用停用状态】=‘启用’，更新当前行【执行状态】为【完成】、【完成时间】默认当前时间，并生成替换关系，记录替换关系id以及行id
 * <p>
 * 2、如《PLM变更单》分录【替换关系】=【新增】,分录【替换件启用停用状态】=‘启用’，更新当前行【执行状态】为【完成】、【完成时间】默认当前时间，并生成替换关系，记录替换关系id以及行id
 * <p>
 * 3、如《PLM变更单》分录【替换关系】=【删除】,分录【启用停用状态】=‘停用’，更新当前行【执行状态】为【完成】、【完成时间】默认当前时间，并生成替换关系，记录替换关系id以及行id
 * @Date: 2024/8/29 15:38
 * @Created: by ZZSLL
 */

public class GenerateReplaceRelationshipTaskEdit extends AbstractTask {

    @Override
    public void execute(RequestContext requestContext, Map<String, Object> map) throws KDException {
        DynamicObject[] plms = queryPLMChange();

        for (DynamicObject plmObj : plms) {
            DynamicObjectCollection collection = plmObj.getDynamicObjectCollection("yem_change_detail");
            DynamicObjectCollection enabled = new DynamicObjectCollection();
            for (DynamicObject plm : collection) {
                String enablesta = plm.getString("yem_enablesta");
                if ("A".equals(enablesta)) {//"替换件启用停用状态 = 启用"
                    enabled.add(plm);
                }
            }
            DynamicObject bomChange = createBOMChange(plmObj, enabled);
            DynamicObjectCollection entry = bomChange.getDynamicObjectCollection("yem_entryentity");
            if (!entry.isEmpty()) {
                DynamicObject replace = null;
                if (entry.size() == 1) {
                    replace = AtlasUtils.Addreplace(bomChange, entry.get(0), null, "yem_em_replace", "add", "PLM");
                }
                for (DynamicObject entryObject : entry) {
                    int idx = entry.indexOf(entryObject);
                    if (idx == 0) continue;
                    replace = AtlasUtils.Addreplace(bomChange, entry.get(0), replace, "yem_em_replace", "update", "PLM");
                }
            }
        }
    }

    private DynamicObject createBOMChange(DynamicObject head, DynamicObjectCollection entries) {
        DynamicObject bomchange = BusinessDataServiceHelper.newDynamicObject("yem_em_bomchange");
        bomchange.set("id", head.getLong("id"));
        bomchange.set("createorg", head.getString("org"));//创建组织-组织
        bomchange.set("yem_producttype", head.getString("yem_producttype"));//产品分类
        bomchange.set("number", head.getString("billno"));//单据编号
        bomchange.set("yem_date", head.getDate("createtime"));//业务日期-创建时间
        bomchange.set("creator", head.getLong("creator.id"));//创建人
        DynamicObjectCollection detail = bomchange.getDynamicObjectCollection("yem_change_detail");
        for (DynamicObject entry : entries) {
            DynamicObject addNew = detail.addNew();
            addNew.set("id", entry.getLong("id"));
            addNew.set("yem_parentno", entry.getString("yem_textfield"));//父件编码
            addNew.set("yem_parentname", entry.getString("yem_name_parent"));//父项名称
            addNew.set("yem_subno", entry.getString("yem_child_before"));//子件编码
            addNew.set("yem_subname", entry.getString("yem_name_child"));//子件名称
            addNew.set("yem_subenname", "");//子件英文名称
            addNew.set("yem_qty", entry.getBigDecimal("yem_qty_before"));//数量
            addNew.set("yem_enablesta", entry.getBigDecimal("yem_enablesta"));//启用停用状态
            addNew.set("yem_replace", entry.getString("yem_exchangetype"));//替换关系
            addNew.set("yem_enableparno", entry.getString("yem_parent_now"));//替换父件编码-父项编码
            addNew.set("yem_enableparname", entry.getString("yem_name_newparent"));//替换父件名称-父项名称
            addNew.set("yem_enablesubno", entry.getString("yem_child_now"));//替换子件编码-子项编码
            addNew.set("yem_enablesubname", entry.getString("yem_name_newchild"));//替换子件名称
            addNew.set("yem_enablesubenname", entry.getString("yem_englishname"));//替换件英文名称-子件英文名称
            addNew.set("yem_enablesubqty", entry.getBigDecimal("yem_qty_new"));//数量
            addNew.set("yem_enableddate", entry.getDate("yem_starttime"));//启用时间
            addNew.set("yem_downdate", entry.getDate("yem_stoptime"));//停用时间
            addNew.set("yem_remark", "");//备注
            addNew.set("yem_isass", entry.getBoolean("yem_isassy"));//是否总成
            addNew.set("yem_isresource", entry.getBoolean("yem_isresource"));//是否资源
        }
        return bomchange;
    }
}
